package flow

type Hooks []Hook

// Hook funcs to handle certain events on the flow
type Hook struct {
	Wait   func(ID string)
	Start  func(ID string)
	Finish func(ID string, res interface{})
	Error  func(ID string, err error)
	Any    func(name string, ID string, extra ...interface{})
}

// Trigger a hook
func (hooks Hooks) Trigger(name string, ID string, extra ...Data) {
	for _, h := range hooks {
		if h.Any != nil {
			h.Any(name, ID, extra...)
		}
		switch name {
		case "Wait":
			if h.Wait != nil {
				h.Wait(ID)
			}
		case "Start":
			if h.Start != nil {
				h.Start(ID)
			}
		case "Finish":
			if h.Finish != nil {
				h.Finish(ID, extra[0])
			}
		case "Error":
			if h.Error != nil {
				h.Error(ID, extra[0].(error))
			}
		}
	}
}

func (hooks Hooks) Wait(ID string)             { hooks.Trigger("Wait", ID) }
func (hooks Hooks) Start(ID string)            { hooks.Trigger("Start", ID) }
func (hooks Hooks) Finish(ID string, res Data) { hooks.Trigger("Finish", ID, res) }
func (hooks Hooks) Error(ID string, err error) { hooks.Trigger("Error", ID, err) }
